<?php
namespace Teknisa\PED\Modules\Pedidos\Factories;

use Doctrine\ORM\EntityManager;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DB\StoredProcedure\Param;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;

class Ped05700importacaoSolicitacaoComprasFactory {
    protected $connection;

    public function __construct(EntityManager $entityManager) {
        $this->connection       = $entityManager->getConnection();
        $this->entityManager    = $entityManager;
    }
     
    public function buscaPrecoBase($cdfilial, $cdcliente, $cdvendedor, $cdmodpgto){
        try{
            $sql = "SELECT BUSCA_PRECOBASE(:P_CDFILIAL, :P_CDCLIENTE, :P_CDVENDEDOR, :P_CDMODPGTO) FROM DUAL";
            
            $params = array(
                'P_CDFILIAL' => $cdfilial, 
                'P_CDCLIENTE' => $cdcliente, 
                'P_CDVENDEDOR' => $cdvendedor, 
                'P_CDMODPGTO' => $cdmodpgto
            );
            
            $std = $this->connection->prepare($sql);
            $std->execute($params);
            return $std->fetchAll();
        }catch(\Exception $e){
            throw $e;
        }
    }
    
    public function buscaitemSoliRelacProdSC($cdfilsoli, $cdfillan, $nrsolicmp, $dtprogentr, $tdtprogentr, $cdfilial){
        try{
            $sql = "SELECT I.CDPRODUTO, P.CDARVPROD, P.NMPRODUTO
                      FROM ITEMSOLI I, PRODUTO P
                     WHERE CDFILSOLI = :P_CDFILSOLI
                       AND CDFILLAN  = :P_CDFILLAN
                       AND NRSOLICMP = :P_NRSOLICMP
                       AND ((I.DTPROGENTR = :P_DTPROGENTR) OR (:P_TDTPROGENTR = 'T'))
                       AND I.CDPRODUTO = P.CDPRODUTO
                       AND PROD_RELAC_SC(:P_CDFILIAL, I.CDPRODUTO) IS NULL";
                       
            $params = array(
                'P_CDFILSOLI' => $cdfilsoli,
                'P_CDFILLAN' => $cdfillan,
                'P_NRSOLICMP' => $nrsolicmp,
                'P_DTPROGENTR' => $dtprogentr,
                'P_TDTPROGENTR' => $tdtprogentr,
                'P_CDFILIAL' => $cdfilial
            );
            
            $std = $this->connection->prepare($sql);
            $std->execute($params);
            return $std->fetchAll();
        }catch(\Exception $e){
            throw $e;
        }
    }

    public function buscaProdFatorDtEnt($cdfilial,$nrsolicmp,$tdtprogentr,$cdfillan,$dtprogentr,$cdfilsoli){
        try{
            $sql = "SELECT I.CDPRODUTO, P.CDARVPROD, P.NMPRODUTO
                    FROM ITEMSOLI I, PRODUTO P
                    WHERE I.CDFILSOLI = :P_CDFILSOLI
                    AND I.CDFILLAN  = :P_CDFILLAN
                    AND I.NRSOLICMP = :P_NRSOLICMP
                    AND I.DTPROGENTR = :P_DTPROGENTR
                    AND PROD_RELAC_SC(:P_CDFILIAL, I.CDPRODUTO) = P.CDPRODUTO
                    AND BUSCA_FATOR_PRECO(:P_CDFATPRECO, PROD_RELAC_SC(:P_CDFILIAL, I.CDPRODUTO)) <= 0";
            
            $params = array(
                'P_CDFILIAL' => $cdfilial, 
                'P_NRSOLICMP' => $nrsolicmp,
                'P_TDTPROGENTR' => $tdtprogentr,
                'P_CDFILLAN' => $cdfillan,
                'P_DTPROGENTR' => $dtprogentr, 
                'P_CDFILSOLI' => $cdfilsoli
            );
            
            $std = $this->connection->prepare($sql);
            $std->execute($params);
            return $std->fetchAll();
        }catch(\Exception $e){
            throw $e;
        }
    }
    
    public function buscaPrecoBaseDtEnt($cdfilial, $nrsolicmp, $tdtprogentr, $cdfillan, $dtprogentr, $cdfilsoli){
        try{
            $sql = "SELECT I.CDPRODUTO, P.CDARVPROD, P.NMPRODUTO
                      FROM ITEMSOLI I, PRODUTO P
                     WHERE CDFILSOLI = :P_CDFILSOLI
                       AND CDFILLAN  = :P_CDFILLAN
                       AND NRSOLICMP = :P_NRSOLICMP
                       AND DTPROGENTR = :P_DTPROGENTR
                       AND PROD_RELAC_SC(:P_CDFILIAL, I.CDPRODUTO) = P.CDPRODUTO
                       AND BUSCA_PRECO_PED(:P_CDFILIAL, P.CDARVPROD, :P_CDCLIENTE, :P_CDPRECOBASE, I.DTPROGENTR, :P_CDMODPGTO) < = 0";
            
            $params = array(
                'P_CDFILIAL' => $cdfilial, 
                'P_NRSOLICMP' => $nrsolicmp,
                'P_TDTPROGENTR' => $tdtprogentr,
                'P_CDFILLAN' => $cdfillan,
                'P_DTPROGENTR' => $dtprogentr, 
                'P_CDFILSOLI' => $cdfilsoli
            );
            
            $std = $this->connection->prepare($sql);
            $std->execute($params);
            return $std->fetchAll();
        }catch(\Exception $e){
            throw $e;
        }
    }
    
    public function buscaItemSoli($cdfilsoli, $cdfillan, $nrsolicmp, $dtprogentr, $tdtprogentr){
        try{
            $sql = "SELECT DISTINCT
                           I.CDPRODUTO, I.DTPROGENTR,
                           NVL(QTSOLIPROD,0) QTSOLIPROD
                      FROM ITEMSOLI I, PRSUBGRPROD PS
                     WHERE CDFILSOLI = :P_CDFILSOLI
                       AND CDFILLAN  = :P_CDFILLAN
                       AND NRSOLICMP = :P_NRSOLICMP
                       AND (PS.CDPRODUTO(+) = I.CDPRODUTO)
                       AND ((PS.CDGRUPPROD = :P_CDGRUPO ) OR (:P_CDGRUPO IS NULL))
                       AND ((INSTR(:P_CDSUBGRPROD, CDSUBGRPROD) > 0) OR (:P_TCDSUBGRPROD ='T'))
                       AND ((I.DTPROGENTR = :P_DTPROGENTR) OR (:P_TDTPROGENTR = 'T'))
                     ORDER BY DTPROGENTR";
            
            $cdgrupo = null;
            $cdsubgrupo = null;
            $tcdsubgrupo = T;

            $params = array(
                'P_CDFILSOLI' => $cdfilsoli,
                'P_CDFILLAN' => $cdfillan,
                'P_NRSOLICMP' => $nrsolicmp,
                'P_CDGRUPO' => $cdgrupo, 
                'P_CDSUBGRPROD'=> $cdsubgrupo,
                'P_TCDSUBGRPROD' => $tcdsubgrupo,
                'P_DTPROGENTR' => $dtprogentr, 
                'P_TDTPROGENTR' => $tdtprogentr
            );
            
            $std = $this->connection->prepare($sql);
            $std->execute($params);
            return $std->fetchAll();
        }catch(\Exception $e){
            throw $e;
        }
    }
    
    public function buscaFator($sgfatpreco, $tsgfatpreco){
        try{
            $sql = "SELECT
                CDFATPRECO, SGFATPRECO, NMFATPRECO
                FROM FATORPRECO
                WHERE (SGFATPRECO = :P_SGFATPRECO OR :P_TSGFATPRECO = 'T')
                ORDER BY SGFATPRECO";
                
            $params = array(
                'P_SGFATPRECO' => $sgfatpreco,
                'P_TSGFATPRECO' => $tsgfatpreco
            );
            
            $std = $this->connection->prepare($sql);
            $std->execute($params);
            return $std->fetchAll();
        }catch(\Exception $e){
            throw $e;
        }
    }
    
    public function buscaProdFator($cdfilial, $cdfilsoli, $cdfillan, $nrsolicmp, $fator){
        try{
            $sql = "SELECT
                    P.CDPRODUTO, P.CDARVPROD, P.NMPRODUTO
                    FROM PRODUTO P,
                    (SELECT DISTINCT
                            PROD_RELAC_SC(:P_CDFILIAL, CDPRODUTO) AS CDPRODUTO
                    FROM ITEMSOLI
                   WHERE CDFILSOLI = :P_CDFILSOLI
                     AND CDFILLAN  = :P_CDFILLAN
                     AND NRSOLICMP = :P_NRSOLICMP
                    MINUS
                    SELECT
                    CDPRODUTO
                    FROM PRODFATOR
                   WHERE CDFATPRECO = :P_CDFATPRECO)X
                   WHERE X.CDPRODUTO = P.CDPRODUTO
                   ORDER BY CDPRODUTO";
    
            $params = array(
                'P_CDFILIAL' => $cdfilial,
                'P_CDFILSOLI' => $cdfilsoli,
                'P_CDFILLAN' => $cdfillan,
                'P_NRSOLICMP' => $nrsolicmp,
                'P_CDFATPRECO' => $fator
            );
            
            $std = $this->connection->prepare($sql);
            $std->execute($params);
            return $std->fetchAll();
        }catch(\Exception $e){
            throw $e;
        }
    }
    
    public function selectPrecoBase($cdfilial, $cdfilsoli, $cdfillan, $nrsolicmp, $stResult){
        try{
            $sql = "SELECT
                       P.CDPRODUTO, P.CDARVPROD, P.NMPRODUTO
                    FROM
                       PRODUTO P ,
                       (
                          SELECT DISTINCT PROD_RELAC_SC(:P_CDFILIAL, CDPRODUTO) AS CDPRODUTO
                          FROM ITEMSOLI
                          WHERE
                             CDFILSOLI = :P_CDFILSOLI
                             AND CDFILLAN  = :P_CDFILLAN
                             AND NRSOLICMP = :P_NRSOLICMP
                          MINUS
                          SELECT
                             CDPRODUTO
                          FROM
                             ITEMPREBASE
                          WHERE
                             CDPRECOBASE = :P_CDPRECOBASE)X
                    WHERE
                       X.CDPRODUTO = P.CDPRODUTO
                    ORDER BY CDPRODUTO";
    
            $params = array(
                'P_CDFILIAL' => $cdfilial,
                'P_CDFILSOLI' => $cdfilsoli,
                'P_CDFILLAN' => $cdfillan,
                'P_NRSOLICMP' => $nrsolicmp,
                'P_CDPRECOBASE' => $stResult
            );
            
            $std = $this->connection->prepare($sql);
            $std->execute($params);
            return $std->fetchAll();
        }catch(\Exception $e){
            throw $e;
        }
    }
    
    public function selectDataEndPed($cdfilsoli, $cdfillan, $nrsolicmp){
        try{
            $sql = "SELECT 
                        MIN(DTPROGENTR) AS DTENTREGA
                    FROM 
                       ITEMSOLI
                    WHERE 
                        CDFILSOLI = :P_CDFILISOLI
                    AND CDFILLAN  = :P_CDFILLAN 
                    AND NRSOLICMP = :P_NRSOLICMP";
    
            $params = array(
                'P_CDFILISOLI' => $cdfilsoli,
                'P_CDFILLAN' => $cdfillan,
                'P_NRSOLICMP' => $nrsolicmp
            );
            
            $std = $this->connection->prepare($sql);
            $std->execute($params);
            return $std->fetchAll();
        }catch(\Exception $e){
            throw $e;
        }
    }
    
    public function selectValidaPreco($cdfilial, $cdfilsoli, $cdfillan, $nrsolicmp, $stResult, $dtentrega){
        try{
            $sql = "SELECT
                   P.CDPRODUTO, P.CDARVPROD, P.NMPRODUTO
                FROM
                   PRODUTO P ,
                   (
                      SELECT DISTINCT PROD_RELAC_SC(:P_CDFILIAL, CDPRODUTO) AS CDPRODUTO
                      FROM ITEMSOLI
                      WHERE
                         CDFILSOLI = :P_CDFILSOLI
                         AND CDFILLAN  = :P_CDFILLAN
                         AND NRSOLICMP = :P_NRSOLICMP
                      MINUS
                      SELECT
                         CDPRODUTO
                      FROM
                         ITEMPREBASE
                      WHERE
                         CDPRECOBASE = :P_CDPRECOBASE
                     AND (DTINIVALIPB <= :P_DTPEDIDO
                     AND :P_DTPEDIDO <= NVL(DTFIMVALIPB, TO_DATE('31/12/9999')))
                )X
                WHERE
                   X.CDPRODUTO = P.CDPRODUTO
                ORDER BY
                   CDPRODUTO";
    
            $params = array(
                'P_CDFILIAL' => $cdfilial,
                'P_CDFILSOLI' => $cdfilsoli,
                'P_CDFILLAN' => $cdfillan,
                'P_NRSOLICMP' => $nrsolicmp,
                'P_CDPRECOBASE' => $stResult, 
                'P_DTPEDIDO' => $dtentrega
            );
            
            $std = $this->connection->prepare($sql);
            $std->execute($params);
            return $std->fetchAll();
        }catch(\Exception $e){
            throw $e;
        }
    }
    
    public function selectValidaPrecoZerado($cdfilial, $cdfilsoli, $cdfillan, $nrsolicmp, $stResult, $dtentrega){
        try{
            $sql = "SELECT
                   P.CDPRODUTO, P.CDARVPROD, P.NMPRODUTO
                FROM
                   PRODUTO P ,
                   (
                      SELECT DISTINCT PROD_RELAC_SC(:P_CDFILIAL, CDPRODUTO) AS CDPRODUTO
                      FROM ITEMSOLI
                      WHERE
                         CDFILSOLI = :P_CDFILSOLI
                         AND CDFILLAN  = :P_CDFILLAN
                         AND NRSOLICMP = :P_NRSOLICMP
                      MINUS
                      SELECT
                         CDPRODUTO
                      FROM
                         ITEMPREBASE
                      WHERE
                         CDPRECOBASE = :P_CDPRECOBASE
                     AND (DTINIVALIPB <= :P_DTPEDIDO
                     AND :P_DTPEDIDO <= NVL(DTFIMVALIPB, TO_DATE('31/12/9999')))
                     AND VRITEPREBASE > 0
                )X
                WHERE
                   X.CDPRODUTO = P.CDPRODUTO
                ORDER BY
                   CDPRODUTO";
    
            $params = array(
                'P_CDFILIAL' => $cdfilial,
                'P_CDFILSOLI' => $cdfilsoli,
                'P_CDFILLAN' => $cdfillan,
                'P_NRSOLICMP' => $nrsolicmp,
                'P_CDPRECOBASE' => $stResult, 
                'P_DTPEDIDO' => $dtentrega
            );
            
            $std = $this->connection->prepare($sql);
            $std->execute($params);
            return $std->fetchAll();
        }catch(\Exception $e){
            throw $e;
        }
    }
    
    public function selectCalcTotImpos(){
        try{
            $sql = "SELECT CDFILIAL,
                           NRPEDIDO,
                           CDIMPOSTO,
                           SUM(VRIMPOSTOITPED) AS VRIMPOSTOTAL,
                           SUM(VRBASECALCITPED) AS VRBASETOTAL
                      FROM IMPITEMPEDIDO
                     WHERE CDFILIAL  = :P_CDFILIAL AND
                           NRPEDIDO  = :P_NRPEDIDO
                     GROUP BY CDFILIAL, NRPEDIDO, CDIMPOSTO";
            
            $params = array(
                'P_CDFILIAL' => $cdfilial, 
                'P_NRPEDIDO' => $cdcliente
            );
            
            $std = $this->connection->prepare($sql);
            $std->execute($params);
            return $std->fetchAll();
        }catch(\Exception $e){
            throw $e;
        }
    }

    public function selectConsImposPedTot(){
        try{
            $sql = "SELECT I.CDFILIAL,
                           I.NRPEDIDO,
                           I.CDIMPOSTO,
                           I.VRTOTIMPPED,
                           I.VRTOTBCALCPED,
                           T.SGIMPOSTO,
                           T.IDTRATIMPO
                      FROM IMPOSNFTOTPED I, IMPOSTO T
                     WHERE I.CDFILIAL  = :P_CDFILIAL  AND
                           I.NRPEDIDO  = :P_NRPEDIDO  AND
                           I.CDIMPOSTO = :P_CDIMPOSTO AND
                           I.CDIMPOSTO = T.CDIMPOSTO";
            
            $params = array(
                'P_CDFILIAL' => $cdfilial, 
                'P_NRPEDIDO' => $nrpedido,
                'P_CDIMPOSTO'=> $cdimposto
            );
            
            $std = $this->connection->prepare($sql);
            $std->execute($params);
            return $std->fetchAll();
        }catch(\Exception $e){
            throw $e;
        }
    }

    public function selectInsItemPedTot(){
        try{
            $sql = "INSERT INTO IMPOSNFTOTPED
                                (CDFILIAL, NRPEDIDO, CDIMPOSTO,
                                 VRTOTIMPPED, VRTOTBCALCPED)
                                VALUES
                                (:CDFILIAL, :NRPEDIDO, :CDIMPOSTO,
                                 :VRTOTIMPPED, :VRTOTBCALCPED)";
            
            $params = array(
                'CDFILIAL' => $cdfilial, 
                'NRPEDIDO' => $nrpedido,
                'CDIMPOSTO'=> $cdimposto,
                'VRTOTIMPPED'=> $vrtotimpped,
                'VRTOTBCALCPED' => $vrtotbcalcped
            );
            
            $std = $this->connection->prepare($sql);
            $std->execute($params);
            return $std->fetchAll();
        }catch(\Exception $e){
            throw $e;
        }
    }

    public function selectUpdItemPedTot(){
        try{
            $sql = "UPDATE IMPOSNFTOTPED
                           SET VRTOTIMPPED  = :VRTOTIMPPED,
                               VRTOTBCALCPED = :VRTOTBCALCPED
                         WHERE CDFILIAL  = :CDFILIAL AND
                               NRPEDIDO  = :NRPEDIDO AND
                               CDIMPOSTO = :CDIMPOSTO ";
            
            $params = array(
                'CDFILIAL' => $cdfilial, 
                'NRPEDIDO' => $nrpedido,
                'CDIMPOSTO'=> $cdimposto,
                'VRTOTIMPPED'=> $vrtotimpped,
                'VRTOTBCALCPED' => $vrtotbcalcped
            );
            
            $std = $this->connection->prepare($sql);
            $std->execute($params);
            return $std->fetchAll();
        }catch(\Exception $e){
            throw $e;
        }
    }

    public function selectExisteSoliCmpDtPed(){
        try{
            $sql = "SELECT COUNT(1) AS QTRETORNO
                      FROM PEDIDO
                     WHERE CDFILLANCA = :P_CDFILLANCA
                       AND CDFILSOLI  = :P_CDFILSOLI
                       AND NRSOLICMP  = :P_NRSOLICMP
                       AND DTENTPED = :P_DTPROGENTR";
            
            $params = array(
                'P_CDFILLANCA' => $cdfillanca, 
                'P_CDFILSOLI' => $cdfilisoli,
                'P_NRSOLICMP'=> $nrsolicmp,
                'P_DTPROGENTR'=> $dtprogentr,
            );
            
            $std = $this->connection->prepare($sql);
            $std->execute($params);
            return $std->fetchAll();
        }catch(\Exception $e){
            throw $e;
        }
    }
    
    public function selectExisteSoliCmpPed(){
        try{
            $sql = "SELECT COUNT(1) AS QTRETORNO
                      FROM PEDIDO
                     WHERE CDFILLANCA = :P_CDFILLANCA
                       AND CDFILSOLI  = :P_CDFILSOLI
                       AND NRSOLICMP  = :P_NRSOLICMP";
            
            $params = array(
                'P_CDFILLANCA' => $cdfillanca, 
                'P_CDFILSOLI' => $cdfilisoli,
                'P_NRSOLICMP'=> $nrsolicmp,
            );
            
            $std = $this->connection->prepare($sql);
            $std->execute($params);
            return $std->fetchAll();
        }catch(\Exception $e){
            throw $e;
        }
    }
    
    public function gravaItempedido() {
        
        try {
            $this->entityManager->getConnection()->beginTransaction();
            $procedure = new StoredProcedure($this->entityManager->getConnection(), 'GRAVA_ITEMPEDIDO');
            
            $procedure->addParam(new Param('P_CDFILIAL' , Param::PARAM_INPUT, $item['CDFILIAL']     , Param::PARAM_TYPE_STR));
            
            $procedure->execute(); 
            $this->entityManager->getConnection()->commit();

        } catch (\Exception $e) {
            $response->setError(new Error("Error. ". $e->getMessage(), 500));
            $this->entityManager->rollback();
            throw $e;
        }
    }



};